#!/usr/bin/python
import mpd

client = mpd.MPDClient()
client.connect("localhost", 6600)

for entry in client.lsinfo("/"):
    print("%s" % entry)
for key, value in client.status().items():
    print("%s: %s" % (key, value))



#Descriptio, file=sys.stderrn
#
#Using this client, one can manipulate and query stickers. The script is essentially a raw interface to the MPD protocol's sticker command, and is used in exactly the same way.
#Examples

## set sticker "foo" to "bar" on "dir/song.mp3"
#sticker.py set dir/song.mp3 foo bar
#
## get sticker "foo" on "dir/song.mp3"
#sticker.py get dir/song.mp3 foo
#
## list all stickers on "dir/song.mp3"
#sticker.py list dir/song.mp3
#
## find all files with sticker "foo" in "dir"
#sticker.py find dir foo
#
## find all files with sticker "foo"
#sticker.py find / foo
#
## delete sticker "foo" from "dir/song.mp3"
#sticker.py delete dir/song.mp3 foo
#
#sticker.py

#! /usr/bin/env python

# Edit these
HOST = "localhost"
PORT = 6600
PASS = None


from optparse import OptionParser
from socket import error as SocketError
from sys import stderr

from mpd import MPDClient, MPDError


ACTIONS = ("get", "set", "delete", "list", "find")


def main(action, uri, name, value):
    client = MPDClient()
    client.connect(HOST, PORT)
    if PASS:
        client.password(PASS)

    if action == "get":
        print(client.sticker_get("song", uri, name))
    if action == "set":
        client.sticker_set("song", uri, name, value)
    if action == "delete":
        client.sticker_delete("song", uri, name)
    if action == "list":
        stickers = client.sticker_list("song", uri)
        for sticker in stickers:
            print(sticker)
    if action == "find":
        matches = client.sticker_find("song", uri, name)
        for match in matches:
            if "file" in match:
                print(match["file"])


if __name__ == "__main__":
    parser = OptionParser(usage="%prog action args", version="0.1",
                          description="Manipulate and query "
                                      "MPD song stickers.")
    options, args = parser.parse_args()

    if len(args) < 1:
        parser.error("no action specified, must be one of: %s" % " ".join(ACTIONS))
    action = args.pop(0)

    if action not in ACTIONS:
        parser.error("action must be one of: %s" % " ".join(ACTIONS))

    if len(args) < 1:
        parser.error("no URI specified")
    uri = args.pop(0)

    if action in ("get", "set", "delete", "find"):
        if len(args) < 1:
            parser.error("no name specified")
        name = args.pop(0)
    else:
        name = None

    if action == "set":
        if len(args) < 1:
            parser.error("no value specified")
        value = args.pop(0)
    else:
        value = None

    try:
        main(action, uri, name, value)
    except SocketError as e:
        print("%s: error with connection to MPD: %s" % \
                (parser.get_prog_name(), e[1]), file=stderr)
    except MPDError as e:
        print("%s: error executing action: %s" % \
                (parser.get_prog_name(), e), file=stderr)


# vim: set expandtab shiftwidth=4 softtabstop=4 textwidth=79:



#! /usr/bin/env python
#
#Introduction
#
#A python program that continuously polls for song info. Demonstrates how and where to handle errors
#Details
#

from mpd import MPDClient, MPDError, CommandError
import sys


class PollerError(Exception):
    """Fatal error in poller."""


class MPDPoller(object):
    def __init__(self, host="localhost", port="6600", password=None):
        self._host = host
        self._port = port
        self._password = password
        self._client = MPDClient()

    def connect(self):
        try:
            self._client.connect(self._host, self._port)
        # Catch socket errors
        except IOError as err:
            errno, strerror = err
            raise PollerError("Could not connect to '%s': %s" %
                              (self._host, strerror))

        # Catch all other possible errors
        # ConnectionError and ProtocolError are always fatal.  Others may not
        # be, but we don't know how to handle them here, so treat them as if
        # they are instead of ignoring them.
        except MPDError as e:
            raise PollerError("Could not connect to '%s': %s" %
                              (self._host, e))

        if self._password:
            try:
                self._client.password(self._password)

            # Catch errors with the password command (e.g., wrong password)
            except CommandError as e:
                # On CommandErrors we have access to the parsed error response
                # split into errno, offset, command and msg.
                raise PollerError("Could not connect to '%s': "
                                  "password commmand failed: [%d] %s" %
                                  (self._host, e.errno, e.msg))

            # Catch all other possible errors
            except (MPDError, IOError) as e:
                raise PollerError("Could not connect to '%s': "
                                  "error with password command: %s" %
                                  (self._host, e))

    def disconnect(self):
        # Try to tell MPD we're closing the connection first
        try:
            self._client.close()

        # If that fails, don't worry, just ignore it and disconnect
        except (MPDError, IOError):
            pass

        try:
            self._client.disconnect()

        # Disconnecting failed, so use a new client object instead
        # This should never happen.  If it does, something is seriously broken,
        # and the client object shouldn't be trusted to be re-used.
        except (MPDError, IOError):
            self._client = MPDClient()

    def poll(self):
        try:
            song = self._client.currentsong()

        # Couldn't get the current song, so try reconnecting and retrying
        except (MPDError, IOError):
            # No error handling required here
            # Our disconnect function catches all exceptions, and therefore
            # should never raise any.
            self.disconnect()

            try:
                self.connect()

            # Reconnecting failed
            except PollerError as e:
                raise PollerError("Reconnecting failed: %s" % e)

            try:
                song = self._client.currentsong()

            # Failed again, just give up
            except (MPDError, IOError) as e:
                raise PollerError("Couldn't retrieve current song: %s" % e)

        # Hurray!  We got the current song without any errors!
        print(song)


def main():
    from time import sleep

    poller = MPDPoller()
    poller.connect()

    while True:
        poller.poll()
        sleep(3)


if __name__ == "__main__":
    import sys

    try:
        main()

    # Catch fatal poller errors
    except PollerError as e:
        print("Fatal poller error: %s" % e, file=sys.stderr)
        sys.exit(1)

    # Catch all other non-exit errors
    except Exception as e:
        print("Unexpected exception: %s" % e, file=sys.stderr)
        sys.exit(1)

    # Catch the remaining exit errors
    except:
        sys.exit(0)


# vim: set expandtab shiftwidth=4 softtabstop=4 textwidth=79:



import asyncio

from mpd.asyncio import MPDClient

async def main():
    print("Create MPD client")
    client = MPDClient()

    # Not necessary, but should not cause any trouble either
    client.disconnect()

    try:
        await client.connect('localhost', 6600)
    except Exception as e:
        print("Connection failed:", e)
        return

    print("Connected to MPD version", client.mpd_version)

    try:
        status = await client.status()
    except Exception as e:
        print("Status error:", e)
        return
    else:
        print("Status success:", status)

    print(list(await client.commands()))

    import time
    start = time.time()
    for x in await client.listall():
        print("sync:", x)
        print("Time to first sync:", time.time() - start)
        break

    start = time.time()
    async for x in client.listall():
        print("async:", x)
        print("Time to first async:", time.time() - start)
        break

    try:
        await client.addid()
    except Exception as e:
        print("An erroneous command, as expected, raised:", e)

    try:
        async for x in client.plchangesposid():
            print("Why does this work?")
    except Exception as e:
        print("An erroneous asynchronously looped command, as expected, raised:", e)

    i = 0
    async for subsystem in client.idle():
        print("Idle change in", subsystem)
        i += 1
        if i > 5:
            print("Enough changes, quitting")
            break

if __name__ == '__main__':
    asyncio.run(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# IMPORTS
from mpd import (MPDClient, CommandError, FailureResponseCode)
from socket import error as SocketError
from sys import exit
from PIL import Image
from io import BytesIO

## SETTINGS
##
HOST = 'localhost'
PORT = '6600'
PASSWORD = False
SONG = ''
###

client = MPDClient()

try:
    client.connect(host=HOST, port=PORT)
except SocketError:
    exit(1)

if PASSWORD:
    try:
        client.password(PASSWORD)
    except CommandError:
        exit(1)

try:
    missing_cover_art = client.albumart("does/not/exist")
except CommandError as error:
    # Asking for media that does not exist or media that has no
    # albumart should raise:
    #   mpd.base.CommandError: [50@0] {albumart} No file exists
    if error.errno is not FailureResponseCode.NO_EXIST:
        raise error

try:
    cover_art = client.readpicture(SONG)
except CommandError:
    exit(1)

if 'binary' not in cover_art:
    # The song exists but has no embedded cover art
    print("No embedded art found!")
    exit(1)

if 'type' in cover_art:
    print("Cover art of type " + cover_art['type'])

with Image.open(BytesIO(cover_art['binary'])) as img:
    img.show()

client.disconnect()

# VIM MODLINE
# vim: ai ts=4 sw=4 sts=4 expandtab



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# IMPORTS
import sys
import pprint

from mpd import (MPDClient, CommandError)
from socket import error as SocketError

HOST = 'localhost'
PORT = '6600'
PASSWORD = False
##
CON_ID = {'host':HOST, 'port':PORT}
##  

## Some functions
def mpdConnect(client, con_id):
    """
    Simple wrapper to connect MPD.
    """
    try:
        client.connect(**con_id)
    except SocketError:
        return False
    return True

def mpdAuth(client, secret):
    """
    Authenticate
    """
    try:
        client.password(secret)
    except CommandError:
        return False
    return True
##

def main():
    ## MPD object instance
    client = MPDClient()
    if mpdConnect(client, CON_ID):
        print('Got connected!')
    else:
        print('fail to connect MPD server.')
        sys.exit(1)

    # Auth if password is set non False
    if PASSWORD:
        if mpdAuth(client, PASSWORD):
            print('Pass auth!')
        else:
            print('Error trying to pass auth.')
            client.disconnect()
            sys.exit(2)

    ## Fancy output
    pp = pprint.PrettyPrinter(indent=4)

    ## Print out MPD stats & disconnect
    print('\nCurrent MPD state:')
    pp.pprint(client.status())

    print('\nMusic Library stats:')
    pp.pprint(client.stats())

    client.disconnect()
    sys.exit(0)

# Script starts here
if __name__ == "__main__":
    main()



#!/usr/bin/env python

import re
import sys
import os.path
from textwrap import TextWrapper
import urllib.request
try:
    from lxml import etree
except ImportError:
    sys.stderr.write("Please install lxml to run this script.")
    sys.exit(1)

DEPRECATED_COMMANDS = []
SCRIPT_PATH = os.path.dirname(os.path.realpath(__file__))


def get_text(elements, itemize=False):
    paragraphs = []
    highlight_elements = ['varname', 'parameter']
    strip_elements = [
            'returnvalue',
            'command',
            'link',
            'footnote',
            'simpara',
            'footnoteref',
            'function'
    ] + highlight_elements
    for element in elements:
        # put "Since MPD version..." in parenthese
        etree.strip_tags(element, "application")
        for e in element.xpath("footnote/simpara"):
            e.text = "(" + e.text.strip() + ")"

        for e in element.xpath("|".join(highlight_elements)):
            e.text = "*" + e.text.strip() + "*"
        etree.strip_tags(element, *strip_elements)
        if itemize:
            initial_indent = "    * "
            subsequent_indent = "      "
        else:
            initial_indent = "    "
            subsequent_indent = "    "
        wrapper = TextWrapper(subsequent_indent=subsequent_indent,
                              initial_indent=initial_indent)
        text = element.text.replace("\n", " ").strip()
        text = re.subn(r'\s+', ' ', text)[0]
        paragraphs.append(wrapper.fill(text))
    return "\n\n".join(paragraphs)


def main(url):
    header_file = os.path.join(SCRIPT_PATH, "commands_header.txt")
    with open(header_file, 'r') as f:
        print(f.read())

    r = urllib.request.urlopen(url)
    tree = etree.parse(r)
    chapter = tree.xpath('/book/chapter[@id="command_reference"]')[0]
    for section in chapter.xpath("section"):
        title = section.xpath("title")[0].text
        print(title)
        print(len(title) * "-")

        print(get_text(section.xpath("para")))
        print("")

        for entry in section.xpath("variablelist/varlistentry"):
            cmd = entry.xpath("term/cmdsynopsis/command")[0].text
            if cmd in DEPRECATED_COMMANDS:
                continue
            subcommand = ""
            args = ""
            begin_optional = False
            first_argument = True

            for arg in entry.xpath("term/cmdsynopsis/arg"):
                choice = arg.attrib.get("choice", None)
                if choice == "opt" and not begin_optional:
                    begin_optional = True
                    args += "["
                if args != "" and args != "[":
                    args += ", "
                replaceables = arg.xpath("replaceable")
                if len(replaceables) > 0:
                    for replaceable in replaceables:
                        args += replaceable.text.lower()
                elif first_argument:
                    subcommand = arg.text
                else:
                    args += '"{}"'.format(arg.text)
                first_argument = False
            if begin_optional:
                args += "]"
            if subcommand != "":
                cmd += "_" + subcommand
            print(".. function:: MPDClient." + cmd + "(" + args + ")")
            description = get_text(entry.xpath("listitem/para"))
            description = re.sub(r':$', r'::', description, flags=re.MULTILINE)

            print("\n")
            print(description)
            print("\n")

            for screen in entry.xpath("listitem/screen | listitem/programlisting"):
                for line in screen.text.split("\n"):
                    print("        " + line)
            for item in entry.xpath("listitem/itemizedlist/listitem"):
                print(get_text(item.xpath("para"), itemize=True))
                print("\n")

if __name__ == "__main__":
    url = "https://raw.githubusercontent.com/MusicPlayerDaemon/MPD/master/doc/protocol.xml"
    if len(sys.argv) > 1:
        url += "?id=release-" + sys.argv[1]
    main(url)



#Multi tag files
#
#Some tag formats (such as ID3v2 and VorbisComment) support defining the same tag multiple times, mostly for when a song has multiple artists. MPD supports this, and sends each occurrence of a tag to the client.
#
#When python-mpd encounters the same tag more than once on the same song, it uses a list instead of a string.
#Function to get a string only song object.


def collapse_tags(song):
   for tag, value in song.iteritems():
       if isinstance(value, list):
           song[tag] = ", ".join(set(value))





from threading import Lock, Thread
from random import choice
from mpd import MPDClient

class LockableMPDClient(MPDClient):
    def __init__(self):
        super(LockableMPDClient, self).__init__()
        self._lock = Lock()

    def acquire(self):
        self._lock.acquire()

    def release(self):
        self._lock.release()

    def __enter__(self):
        self.acquire()

    def __exit__(self, type, value, traceback):
        self.release()


client = LockableMPDClient()
client.connect("localhost", 6600)
# now whenever you need thread-safe access
# use the 'with' statement like this:
with client: # acquire lock
    status = client.status()
# if you leave the block, the lock is released
# it is recommend to leave it soon,
# otherwise your other threads will blocked.

# Let's test if it works ....
def fetch_playlist():
    for i in range(10):
        if choice([0, 1]) == 0:
            with client:
                song = client.currentsong()
            assert isinstance(song, dict)
        else:
            with client:
                playlist = client.playlist()
            assert isinstance(playlist, list)


threads = []
for i in range(5):
    t = Thread(target=fetch_playlist)
    threads.append(t)
    t.start()
for t in threads:
    t.join()

print("Done...")



from __future__ import print_function
from mpd import MPDProtocol
from twisted.internet import protocol
from twisted.internet import reactor


class MPDApp(object):
    # Example application which deals with MPD

    def __init__(self, protocol):
        self.protocol = protocol

    def __call__(self, result):
        # idle result callback
        print('Subsystems: {}'.format(list(result)))

        def status_success(result):
            # status query success
            print('Status success: {}'.format(result))

        def status_error(result):
            # status query failure
            print('Status error: {}'.format(result))

        # query player status
        self.protocol.status()\
            .addCallback(status_success)\
            .addErrback(status_error)


class MPDClientFactory(protocol.ClientFactory):
    protocol = MPDProtocol

    def buildProtocol(self, addr):
        print('Create MPD protocol')
        protocol = self.protocol()
        protocol.factory = self
        protocol.idle_result = MPDApp(protocol)
        return protocol

    def clientConnectionFailed(self, connector, reason):
        print('Connection failed - goodbye!: {}'.format(reason))
        reactor.stop()

    def clientConnectionLost(self, connector, reason):
        print('Connection lost - goodbye!: {}'.format(reason))
        if reactor.running:
            reactor.stop()


if __name__ == '__main__':
    factory = MPDClientFactory()
    reactor.connectTCP('localhost', 6600, factory)
    reactor.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# IMPORTS
from mpd import (MPDClient, CommandError)
from random import choice
from socket import error as SocketError
from sys import exit


## SETTINGS
##
HOST = 'localhost'
PORT = '6600'
PASSWORD = False
###


client = MPDClient()

try:
    client.connect(host=HOST, port=PORT)
except SocketError:
    exit(1)

if PASSWORD:
    try:
        client.password(PASSWORD)
    except CommandError:
        exit(1)

client.add(choice(client.list('file')))
client.disconnect()

# VIM MODLINE
# vim: ai ts=4 sw=4 sts=4 expandtab
